@@ -27,10 +27,12 @@
 
 #include <glib.h>
 #include <libgweather/gweather-timezone.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
 typedef struct _GWeatherLocation GWeatherLocation;
+typedef gboolean (* GWeatherFilterFunc) (GWeatherLocation* location, gpointer user_data);
 
 typedef enum { /*< underscore_name=gweather_location_level >*/
     GWEATHER_LOCATION_WORLD,
@@ -73,6 +75,21 @@ GWeatherLocation      *gweather_location_find_nearest_city (GWeatherLocation *lo
 							    double            lat,
 							    double            lon);
 
+GWeatherLocation      *gweather_location_find_nearest_city_full (GWeatherLocation  *loc,
+								 double             lat,
+								 double             lon,
+								 GWeatherFilterFunc func,
+								 gpointer           user_data,
+								 GDestroyNotify     destroy);
+
+void                  gweather_location_detect_nearest_city (GWeatherLocation   *loc,
+							     double              lat,
+							     double              lon,
+							     GCancellable       *cancellable,
+							     GAsyncReadyCallback callback,
+							     gpointer            user_data);
+GWeatherLocation      *gweather_location_detect_nearest_city_finish (GAsyncResult *result, GError **error);
+
 const char            *gweather_location_get_country    (GWeatherLocation  *loc);
 
 GWeatherTimezone      *gweather_location_get_timezone   (GWeatherLocation  *loc);
